SystemOrganization addCategory: #'Soup-Tests-Core'!

TestCase subclass: #SoupDecoderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Soup-Tests-Core'!

!SoupDecoderTest methodsFor: 'as yet unclassified' stamp: 'za 12/30/2008 03:09'!
testSmartQuotesNotSoSmartAnymore
	| decoder |
	decoder := SoupDecoder new.
	decoder convertSmartQuotesToHtml.
	decoder string: ('{1}Foo{2} <!!-- blah -->' format: (#(145 146) collect: [:e | e asCharacter])).
	self assert: (decoder decode = '&lsquo;Foo&rsquo; <!!-- blah -->')! !

TestCase subclass: #SoupElementTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Soup-Tests-Core'!

!SoupElementTest methodsFor: 'accessing' stamp: 'sd 12/11/2010 15:17'!
testAccessingText
	"The method text returns all the texts elements contained"
	"self debug: #testAccessingText"

	| doc |
	doc := '[''<html><head><title>Page title</title></head> <body><p id="firstpara" align="center">This is paragraph <b>one</b><p id="secondpara" align="blah">This is paragraph <b>two</b></html>]'.
	self assert: (Soup fromString: doc ) text = '[''Page title This is paragraph oneThis is paragraph two]' ! !

!SoupElementTest methodsFor: 'finding strings' stamp: 'za 12/22/2008 22:15'!
testFindNextSiblingString
	| soup b | 
	soup := Soup fromString: '<blockquote id="1"><blockquote id="1.1"></blockquote></blockquote><blockquote id="2"><blockquote id="2.1"></blockquote></blockquote><hr/>abc<br/>def<hr/><blockquote id="3"><blockquote id="3.1"></blockquote></blockquote><blockquote id="4">'.
	b := soup findTag: [:e | e id = '2'].
	self assert: (b findNextSiblingString: true) contents = 'abc'

! !

!SoupElementTest methodsFor: 'finding strings' stamp: 'za 12/22/2008 20:17'!
testFindNextSiblingStrings
	| soup b | 
	soup := Soup fromString: '<blockquote id="1"><blockquote id="1.1"></blockquote></blockquote><blockquote id="2"><blockquote id="2.1"></blockquote></blockquote>abc<br/>def<blockquote id="3"><blockquote id="3.1"></blockquote></blockquote><blockquote id="4">'.
	b := soup findTag: [:e | e id = '2'].
	self assert: (b findNextSiblingStrings: true) size = 2.

! !

!SoupElementTest methodsFor: 'finding tags' stamp: 'za 12/22/2008 22:18'!
testFindNextSiblingTag
	| soup b | 
	soup := Soup fromString: '<blockquote id="1"><blockquote id="1.1"></blockquote></blockquote><blockquote id="2"><blockquote id="2.1"></blockquote></blockquote>abc<blockquote id="3"><blockquote id="3.1"></blockquote></blockquote><blockquote id="4">'.
	b := soup findTag: [:e | e id = '2'].
	self assert: (b findNextSiblingTag: true) id = '3'

! !

!SoupElementTest methodsFor: 'finding tags' stamp: 'sd 12/11/2010 17:38'!
testFindNextSiblingTags
	"self debug: #testFindNextSiblingTags"
	| soup b | 
	soup := Soup fromString: '<blockquote id="1"><blockquote id="1.1"></blockquote></blockquote><blockquote id="2"><blockquote id="2.1"></blockquote></blockquote><blockquote id="3"><blockquote id="3.1"></blockquote></blockquote><blockquote id="4">'.
	b := soup findTag: [:e | e id = '2'].
	self assert: (b findNextSiblingTags: true) size = 2.
	"Two because we get the node 3 (with the subnode 3.1 and the node 4"
	self assert: (b findNextSiblingTags: [:e | e id = '4']) size = 1.
	! !

!SoupElementTest methodsFor: 'finding tags' stamp: 'sd 1/7/2011 14:33'!
testFindNextSiblingVersusFindTags
	"self debug: #testFindNextSiblingVersusFindTags"
	| soup b | 
	soup := Soup fromString: '<blockquote id="1"><blockquote id="1.1"></blockquote></blockquote><blockquote id="2"><blockquote id="2.1"></blockquote></blockquote><blockquote id="3"><blockquote id="3.1"></blockquote></blockquote><blockquote id="4">'.
	b := soup findTag: [:e | e id = '2'].
	self assert: (b findNextSiblingTags: [:e | '3*' match: e id]) size = 1.
	"here we see the difference between siblings and not. There are two nodes
	matching the id 3. 3 and 3.1 but they are not on the same level."
	self assert: (b findNextTags: [:e | '3*' match: e id]) size = 2.! !

!SoupElementTest methodsFor: 'finding tags' stamp: 'za 12/22/2008 22:28'!
testFindNextString
	| soup b |
	soup := Soup fromString: '<b id="1"><b id="2">abc<b id="3"><b id="4">'.
	b := soup findTag: [:e | e id = '2'].
	self assert: (b findNextString: true) contents = 'abc'.

	! !

!SoupElementTest methodsFor: 'finding tags' stamp: 'za 12/22/2008 22:27'!
testFindNextStrings
	| soup b | 
	soup := Soup fromString: '<b id="1"><b id="2">abc<b id="3">def<b id="4">'.
	b := soup findTag: [:e | e name = 'b' and: [e id = '2']].
	self assert: (b findNextStrings: true) size = 2.

! !

!SoupElementTest methodsFor: 'finding tags' stamp: 'za 12/22/2008 22:20'!
testFindNextTag
	| soup b |
	soup := Soup fromString: '<b id="1"><b id="2">abc<b id="3"><b id="4">'.
	b := soup findTag: [:e | e id = '2'].
	self assert: (b findNextTag: 'b') id = '3'.

	! !

!SoupElementTest methodsFor: 'finding tags' stamp: 'za 12/22/2008 22:20'!
testFindNextTags
	| soup b | 
	soup := Soup fromString: '<b id="1"><b id="2">abc<b id="3"><b id="4">'.
	b := soup findTag: [:e | e name = 'b' and: [e id = '2']].
	self assert: (b findNextTags: 'b') size = 2.
	self assert: (b findNextTags: [:e | e name = 'b' and: [e id = '4']]) size = 1.
! !

!SoupElementTest methodsFor: 'finding strings' stamp: 'za 12/22/2008 22:14'!
testFindPreviousSiblingString
	| soup b | 
	soup := Soup fromString: '<blockquote id="1"><blockquote id="1.1"></blockquote></blockquote><blockquote id="2"><blockquote id="2.1"></blockquote></blockquote>abc<br/>def<blockquote id="3"><blockquote id="3.1"></blockquote></blockquote><blockquote id="4">'.
	b := soup findTag: [:e | e id = '3'].
	self assert: (b findPreviousSiblingString: true) contents = 'def'

! !

!SoupElementTest methodsFor: 'finding strings' stamp: 'za 12/22/2008 20:22'!
testFindPreviousSiblingStrings
	| soup b | 
	soup := Soup fromString: '<blockquote id="1"><blockquote id="1.1"></blockquote></blockquote><blockquote id="2"><blockquote id="2.1"></blockquote></blockquote>abc<br/>def<blockquote id="3"><blockquote id="3.1"></blockquote></blockquote><blockquote id="4">'.
	b := soup findTag: [:e | e id = '3'].
	self assert: (b findPreviousSiblingStrings: true) size = 2.

! !

!SoupElementTest methodsFor: 'finding tags' stamp: 'za 12/22/2008 22:12'!
testFindPreviousSiblingTag
	| soup b | 
	soup := Soup fromString: '<blockquote id="1"><blockquote id="1.1"></blockquote></blockquote><blockquote id="2"><blockquote id="2.1"></blockquote></blockquote><blockquote id="3"><blockquote id="3.1"></blockquote></blockquote><blockquote id="4">'.
	b := soup findTag: [:e | e id = '3'].
	self assert: (b findPreviousSiblingTag: true) id = '2'

! !

!SoupElementTest methodsFor: 'finding tags' stamp: 'za 12/22/2008 20:24'!
testFindPreviousSiblingTags
	| soup b | 
	soup := Soup fromString: '<blockquote id="1"><blockquote id="1.1"></blockquote></blockquote><blockquote id="2"><blockquote id="2.1"></blockquote></blockquote><blockquote id="3"><blockquote id="3.1"></blockquote></blockquote><blockquote id="4">'.
	b := soup findTag: [:e | e id = '3'].
	self assert: (b findPreviousSiblingTags: true) size = 2.
	self assert: (b findPreviousSiblingTags: [:e | e id = '1']) size = 1.
! !

!SoupElementTest methodsFor: 'finding tags' stamp: 'za 12/22/2008 22:28'!
testFindPreviousString
	| soup b |
	soup := Soup fromString: '<b id="1"><b id="2">abc<b id="3"><b id="4">'.
	b := soup findTag: [:e | e id = '3'].
	self assert: (b findPreviousString: true) contents = 'abc'.

	! !

!SoupElementTest methodsFor: 'finding tags' stamp: 'za 12/22/2008 22:32'!
testFindPreviousStrings
	| soup b | 
	soup := Soup fromString: '<b id="1"><b id="2">def<br/>abc<hr/></b><b id="3"><b id="4">'.
	b := soup findTag: 'hr'.
	self assert: (b findPreviousStrings: true) size = 2.

! !

!SoupElementTest methodsFor: 'finding tags' stamp: 'za 12/22/2008 22:20'!
testFindPreviousTag
	| soup b |
	soup := Soup fromString: '<b id="1"><b id="2">abc<b id="3"><b id="4">'.
	b := soup findTag: [:e | e id = '3'].
	self assert: (b findPreviousTag: 'b') id = '2'.

	! !

!SoupElementTest methodsFor: 'finding tags' stamp: 'za 12/22/2008 22:21'!
testFindPreviousTags
	| soup b | 
	soup := Soup fromString: '<b id="1"><b id="2">abc<b id="3"><b id="4">'.
	b := soup findTag: [:e | e name = 'b' and: [e id = '3']].
	self assert: (b findPreviousTags: 'b') size = 2.
	self assert: (b findPreviousTags: [:e | e name = 'b' and: [e id = '1']]) size = 1.
! !

!SoupElementTest methodsFor: 'accessing' stamp: 'za 12/26/2008 22:33'!
testNextSibling
	| soup li |
	soup := Soup fromString: '<ul><li>1<p>A</p>B<li>2<li>3</ul>'.
	li := (soup findTag: 'li') nextSibling.
	self assert: li name = 'li'.
	self assert: li string = '2'.
	self assert: (soup findString: '1') nextSibling name = 'p'.
	self assert: (soup findTag: 'p') nextSibling contents = 'B'.
	self assert: (soup findTag: 'p') nextSibling previousSibling nextSibling contents = 'B'.! !

SoupElementTest subclass: #SoupTagTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Soup-Tests-Core'!

!SoupTagTest methodsFor: 'accessing' stamp: 'za 12/22/2008 17:37'!
testAddChild
	| tag childOne childTwo |
	tag := SoupTag new.
	childOne := SoupTag new.
	childTwo := SoupString new.
	tag addChild: childOne.
	self assert: (tag childAt: 1) = childOne.
	tag addChild: childTwo.
	self assert: (tag childAt: 1) = childOne.
	self assert: (tag childAt: 2) = childTwo.
	self should: [tag childAt: 3] raise: Error.
	! !

!SoupTagTest methodsFor: 'accessing' stamp: 'za 12/22/2008 17:39'!
testAttributeAt
	| tag |
	tag := SoupTag new.
	tag attributeAt: 'abc' put: 'def'.
	self assert: (tag attributeAt: 'abc') = 'def'.
	self should: (tag attributeAt: 'xyz') = nil! !

!SoupTagTest methodsFor: 'accessing' stamp: 'za 12/22/2008 17:40'!
testAttributeAtPut
	self testAttributeAt ! !

!SoupTagTest methodsFor: 'accessing' stamp: 'sd 1/7/2011 14:34'!
testBeParentOf
	| parent childOne childTwo |
	parent := SoupTag new.
	childOne := SoupTag new.
	childTwo := SoupString new.
	parent beParentOf: childOne.
	parent beParentOf: childTwo.
	self assert: childOne nextSibling = childTwo.
	self assert: childTwo previousSibling = childOne.
	parent childrenDo: [ :anElement |
		self assert: ({childOne. childTwo.} includes: anElement) ].! !

!SoupTagTest methodsFor: 'accessing' stamp: 'za 12/22/2008 17:45'!
testBeSelfClosing
	self testIsSelfClosing! !

!SoupTagTest methodsFor: 'accessing' stamp: 'za 12/22/2008 17:45'!
testChildAt
	self testAddChild! !

!SoupTagTest methodsFor: 'enumerating' stamp: 'sd 12/5/2010 23:12'!
testChildStringsDo
	"self debug: #testChildStringsDo"
	"childStringsDo: evaluates a block only on leaves which are plain strings"
	| soup stringsRemaining | 
	soup := Soup fromString: '<p>xyz</p>abc<span>123</span><br/>def'.
	stringsRemaining := #(abc def) asOrderedCollection.
	soup childStringsDo: 
				[ :aString |
				self assert: aString isString.
				stringsRemaining remove: aString contents ].
	self assert: stringsRemaining isEmpty.
	
	stringsRemaining := #(xyz) asOrderedCollection.
	soup := Soup fromString: '<p>xyz</p>'.
	"While xyz was not considered as a string in the previous text, it is now one since we accessed it using message p."
	soup p childStringsDo: 
				[ :aString |
				self assert: aString isString.
				stringsRemaining remove: aString contents ].
	self assert: stringsRemaining isEmpty.
	
	stringsRemaining := #(xyz) asOrderedCollection.
	soup := Soup fromString: '<b>xyz</b>'.
	soup childStringsDo: 
				[ :aString |
				self assert: aString isString.
				stringsRemaining remove: aString contents ].
	"Here b is decorated so it is not iterated"
	self deny: stringsRemaining isEmpty.
	
	stringsRemaining := #(xyz) asOrderedCollection.
	soup := Soup fromString: '<b>xyz</b>'.
	"Here we access the string using the message b so it is iterated."
	soup b childStringsDo: 
				[ :aString |
				self assert: aString isString.
				stringsRemaining remove: aString contents ].
	self assert: stringsRemaining isEmpty.
	
	! !

!SoupTagTest methodsFor: 'enumerating' stamp: 'sd 12/13/2010 21:39'!
testChildTagsDo
	| soup tagsRemaining | 
	soup := Soup fromString: '<p>abc</p><span>def</span><br/>'.
	tagsRemaining := #(p span br) asOrderedCollection.
	soup childTagsDo: [ :aTag |
		self assert: aTag isTag.
		tagsRemaining remove: aTag name ].
	self assert: tagsRemaining isEmpty
	! !

!SoupTagTest methodsFor: 'enumerating' stamp: 'sd 12/13/2010 21:38'!
testChildrenDo
	| tag childOne childTwo |
	tag := SoupTag new.
	childOne := SoupTag new.
	childTwo := SoupString new.
	tag addChild: childOne.
	tag addChild: childTwo.
	tag childrenDo: [ :anElement |
		self assert: ({childOne. childTwo} includes: anElement) ].! !

!SoupTagTest methodsFor: 'enumerating' stamp: 'sd 12/13/2010 21:47'!
testChildrenDoWorksOnlyAtOneLevel
	"self debug: #testChildrenDoWorksOnlyAtOneLevel"
	| fatherTag grandFatherTag childOne childTwo |
	grandFatherTag := SoupTag new.
	fatherTag := SoupTag new.
	childOne := SoupTag new.
	childTwo := SoupString new.
	grandFatherTag addChild: fatherTag.
	fatherTag addChild: childOne.
	fatherTag addChild: childTwo.
	grandFatherTag childrenDo: [ :anElement |
		self deny: ({childOne. childTwo} includes: anElement) ].
	grandFatherTag childrenDo: [ :anElement |
		self assert: ({fatherTag} includes: anElement) ].! !

!SoupTagTest methodsFor: 'error handling' stamp: 'za 12/21/2008 14:18'!
testDoesNotUnderstand
	| soup body |
	soup := Soup fromString: '<body id="a"><p id="b">a<span>b</span><br/>'.
	body := soup findTag: 'body'.
	self assert: body id = 'a'.
	self assert: body p id = 'b'.
	self assert: body br name = 'br'.
	self assert: body missing isNil.
	self should: [body test: 1] raise: MessageNotUnderstood ! !

!SoupTagTest methodsFor: 'finding strings' stamp: 'sd 12/13/2010 22:03'!
testFindAllStrings
	"self run: #testFindAllStrings"
	"Returns all the strings equal to the argument. Note that the argument can be a regex to match several elements." 
	
	| soup |
	soup := Soup fromString: '<p>abcxyz<span>abcd</span>abc</p>'.
	self assert: ((soup findAllStrings: 'abcd') size = 1).
	self assert: ((soup findAllStrings: 'abc' asRegex) size = 3).

! !

!SoupTagTest methodsFor: 'finding strings' stamp: 'sd 12/13/2010 22:05'!
testFindAllStringsWithLargerTree
	"self run: #testFindAllStringsWithLargerTree"
	"Returns all the strings equal to the argument. Note that the argument can be a regex to match several elements." 
	
	| soup |
	soup := Soup fromString: '<p>abc<span>def</span></p><b>abcdef<span>defi</span><span>defi2</span></b><p>abc3<span>def</span></p>'.
	self assert: ((soup findAllStrings: 'abcdef' asRegex) size = 1).
	self assert: ((soup findAllStrings: 'abc3') size = 1).
	self assert: ((soup findAllStrings: 'abc' asRegex) size = 3).

! !

!SoupTagTest methodsFor: 'finding tags' stamp: 'za 12/22/2008 19:25'!
testFindAllTags
	| soup |
	soup := Soup fromString: '<body><p>abc<br id="1"/>def<br id="2"/><span class="c">xyz</span></p></body>'.
	soup findAllTags: [:e | self assert: e isTag. true].
	self assert: (soup findAllTags: [:e | true]) size = 5.
	self assert: (soup findAllTags: [:e | false]) size = 0.	
! !

!SoupTagTest methodsFor: 'finding strings' stamp: 'za 12/22/2008 19:07'!
testFindString
	| soup tag |
	soup := Soup fromString: '<p>abc<span>def</span></p>'.
	tag := soup p.
	self assert: (tag findString: 'def') contents = 'def'.
	self assert: (tag findString: #('def')) contents = 'def'.
	self assert: (tag findString: 'def' asRegex) contents = 'def'.
! !

!SoupTagTest methodsFor: 'finding strings' stamp: 'StephaneDucasse 1/7/2011 15:55'!
testFindString2
	<publicTest>
	<systemHelp>
	
	| soup tag |
	"Now we build a soup element with two branches containing different strings."
	soup := Soup fromString: '<p>abc<span>def</span></p><b>abcdef<span>defi</span><span>defi2</span></b>'.
	
	"We are accessing the first nodes being a paragraph"
	tag := soup p.
	
	"Now we can check that the string 'def' is found either directly or among a list of elements to be found, or using a regex"
	self assert: (tag findString: 'def') contents = 'def'.
	self assert: (tag findString: #('def')) contents = 'def'.
	self assert: (tag findString: 'def' asRegex) contents = 'def'.
	
	"Now we progress and query the b element"
	tag := soup b.
	
	"This branch does not contain a 'def' string so it returns nil"
	self assert: (tag findString: 'def') isNil.
		"IMplementation note: returning nil is suboptimal"
	
	"still we can find other string elements using the same api than above"	
	self assert: (tag findString: 'defi') contents = 'defi'.
	self assert: (tag findString: #('defi' 'defi2')) contents = 'defi'.
	"we get the first occurrence here defi"
	
	"We get now the first occurrence matching the regex"
	self assert: (tag findString: '.def' asRegex) contents = 'abcdef'.
	! !

!SoupTagTest methodsFor: 'finding tags' stamp: 'sd 12/11/2010 16:06'!
testFindTag
	
	| soup body |
	soup := Soup fromString: '<body><p>abc<br id="1"/>def<br id="2"/><span class="c">xyz</span></p></body>'.
	body := soup body.
	self assert: (body findTag: 'p') name = 'p'.
	self assert: (body findTag: #(p br)) name = 'p'.
	self assert: (body findTag: [:aTag | aTag name = 'br']) id = '1'.
	self assert: (body findTag: [:aTag | aTag id = '2']) name = 'br'.
	self assert: (body span) name = 'span'
! !

!SoupTagTest methodsFor: 'finding tags' stamp: 'za 12/21/2008 14:07'!
testFindTagIfPresent
	| soup body found |
	soup := Soup fromString: '<body><p>abc<br id="1"/>def<br id="2"/><span class="c">xyz</span></p></body>'.
	body := soup body.
	body findTag: 'foo' ifPresent: [self fail].
	body findTag: #(foo bar) ifPresent: [self fail].
	body findTag: [:aTag | aTag name = 'foo'] ifPresent: [self fail].
	body findTag: [:aTag | aTag id = 'bar'] ifPresent: [self fail].
	found := nil.
	body findTag: 'span' ifPresent: [:aTag | found := aTag].
	self assert: (found attributeAt: 'class') = 'c'.
	found := nil.
	body findTag: #(span) ifPresent: [:aTag | found := aTag].
	self assert: (found attributeAt: 'class') = 'c'.
	found := nil.
	body findTag: [:aTag | aTag name = 'span'] ifPresent: [:aTag | found := aTag].
	self assert: (found attributeAt: 'class') = 'c'.
	found := nil.
	body findTag: [:aTag | aTag id = '2'] ifPresent: [:aTag | found := aTag].
	self assert: found name = 'br'
	! !

!SoupTagTest methodsFor: 'testing' stamp: 'sd 12/13/2010 21:27'!
testIfTag
	"ifTag: executes its block argument when the receiver is a tag."
	
	| tag |
	tag := SoupTag new.
	tag ifTag: [^ self].
	self fail! !

!SoupTagTest methodsFor: 'testing' stamp: 'za 12/22/2008 17:42'!
testIsSelfClosing
	| tag |
	tag := SoupTag new.
	self assert: tag isSelfClosing not.
	tag beSelfClosing.
	self assert: tag isSelfClosing.
	! !

!SoupTagTest methodsFor: 'testing' stamp: 'za 12/22/2008 17:41'!
testIsTag
	| tag |
	tag := SoupTag new.
	self assert: tag isTag.
	! !

!SoupTagTest methodsFor: 'accessing' stamp: 'za 12/22/2008 17:40'!
testName
	| tag |
	tag := SoupTag new.
	tag name: 'abc'.
	self assert: tag name = 'abc'! !

!SoupTagTest methodsFor: 'printing' stamp: 'sd 12/13/2010 21:30'!
testPrintOn
	| tag |
	tag := SoupTag named: 'p'.
	self assert: tag printString = '<p></p>'.
	tag attributeAt: 'id' put: 'a'.
	self assert: tag printString = '<p id="a"></p>'.
	tag attributeAt: 'class' put: 'b'.	
	self assert: tag printString = '<p class="b" id="a"></p>'.
	tag addChild: (SoupTag named: 'span').
	self assert: tag printString = '<p class="b" id="a"><span></span></p>'.
	(tag childAt: 1) addChild: (SoupString new contents: 'abc'; yourself).
	self assert: tag printString = '<p class="b" id="a"><span>abc</span></p>'.
	tag addChild: (SoupTag named: 'br') beSelfClosing; yourself.
	self assert: tag printString = '<p class="b" id="a"><span>abc</span><br /></p>'.		
	
	! !

!SoupTagTest methodsFor: 'accessing' stamp: 'sd 12/5/2010 23:01'!
testSize
	"The size of an element is its number of direct children"
	"self debug: #testSize"
	| soup doc|
	soup := Soup fromString: '<top>1<b>2</b>3</top><hr/>'.
	self assert: (soup size = 2).
	
	doc := '<html><head><title>Page title</title></head><body><p id="firstpara" align="center">This is paragraph <b>one</b><p id="secondpara" align="blah">This is paragraph <b>two</b></html>'.
	soup := Soup fromString: doc.
	self assert: soup size = 1.
	"<html> ....</html> is one node"
	self assert: soup html size = 2.
	"inside we get <head> and <body>..."
	! !

!SoupTagTest methodsFor: 'accessing' stamp: 'sd 12/5/2010 22:54'!
testSizeContainingSquareBrackets
	"The size of an element is its number of direct children"
	"self debug: #testSizeContainingSquareBrackets"
	| doc |
	doc := '[''<html><head><title>Page title</title></head> 
				<body><p id="firstpara" align="center">This is paragraph <b>one</b><p id="secondpara" align="blah">This is paragraph <b>two</b></html>]'.
	(Soup fromString: doc ) size = 3.
	"The document is composed of [ ] and the <...>"! !

!SoupTagTest methodsFor: 'accessing' stamp: 'sd 12/5/2010 22:41'!
testString
	"self debug: #testString"
	| soup |
	soup := Soup fromString: '<b>foo</b>'.
	self assert: (soup b string = 'foo').
	
	"if the element is composed of subnodes return nil"
	soup := Soup fromString: '<b>f<i>e</i>o</b>'.
	self assert: (soup b string) isNil.
	
	soup := Soup fromString: '<b>foo</b><b>bar</b>'.
	self assert: (soup b string = 'foo').
	self assert: (soup b nextSibling string = 'bar').
	! !

TestCase subclass: #SoupParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Soup-Tests-Core'!

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/24/2008 16:02'!
assertSoupFrom: aString printsAs: anotherString
	| soup |
	soup := Soup fromString: aString.
	self assert: soup printString = anotherString! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/25/2008 06:08'!
parseOtherDeclaration: anInteger! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/23/2008 06:06'!
testBadNestedTables
	| soup |
	soup := Soup fromString: '<table><tr><table><tr id=''nested''>'.
	self assert: soup table tr table tr id = 'nested'! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/25/2008 07:12'!
testCData
	| soup data |
	soup := Soup fromString: '<root>foo<!![CDATA[foobar]]>bar</root>'.
	data := soup findString: 'foo.*bar' asRegex. 
	self assert: data contents = 'foobar'.
	self assert: data printString = '<!![CDATA[foobar]]>'! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/25/2008 07:05'!
testComments
	| soup xml |
	xml := 'foo<!!--foobar-->baz'.
	soup := Soup fromString: xml.
	self assert: soup printString = xml.
	self assert: (soup findString: 'foobar') isComment
	! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/25/2008 06:07'!
testDeclaration
	| xml |
	xml := 'foo<!!DOCTYPE foobar>baz'.
	self assertSoupFrom: xml printsAs: xml.
	! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/25/2008 06:04'!
testEntitiesInAttributeValues
	| uri link |
	self assertSoupFrom: '<x t="x&#241;">' printsAs: '<x t="x"></x>'.
	self assertSoupFrom: '<x t="x&#xf1;">' printsAs: '<x t="x"></x>'.
	uri := 'http://crummy.com?sacr&eacute;&amp;bleu'.
	link := '<a href="{1}"></a>' format: uri.
	self assertSoupFrom: link printsAs: link.

! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/24/2008 17:44'!
testEntityConversions
	| text soup |
	text := '&lt;&lt;sacr&eacute;&#32;bleu!!&gt;&gt;'.
	soup := Soup fromString: text.
	self assert: soup printString = text.! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/25/2008 06:53'!
testEntityReplacement
	self assertSoupFrom: '<b>hello&nbsp;there</b>' printsAs: '<b>hello&nbsp;there</b>'! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/25/2008 06:52'!
testIncompleteDeclaration
	" BeautifulSoup will parse this as 'a<!!b <p>c'. It simply gives up after 
	failing to parse a doctype. I can't see the logic of this. Hence: "
	self assertSoupFrom: 'a<!!b <p>c' printsAs: 'a<!!b <p>c</p>'! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/24/2008 19:48'!
testJunkInDeclaration
	self 
		assertSoupFrom: '<!! Foo = -8>a'
		printsAs: '<!!Foo = -8>a'! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/23/2008 02:43'!
testLiteralMode
	| soup |
	soup := Soup fromString: '<script>if (i<imgs.length)</script><b>Foo</b>'.
	self assert: soup script string = 'if (i<imgs.length)'.
	self assert: soup b string = 'Foo'.
	! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/25/2008 01:24'!
testNakedAmpersands
	| invalidUrl validUrl |
	invalidUrl := '<a href="http://example.org?a=1&b=2;3">foo</a>'.
	validUrl := invalidUrl copyReplaceAll: '&'  with: '&amp;'.
	self assertSoupFrom: invalidUrl printsAs: validUrl.
	self assertSoupFrom: validUrl printsAs: validUrl.
	
! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/23/2008 06:05'!
testNestedTables
	| soup | 
	soup := Soup fromString: '<table id="1"><tr><td>Here''s another table:
        <table id="2"><tr><td>Juicy text</td></tr></table></td></tr></table>'.
	self assert: soup table table td string = 'Juicy text'.
	self assert: (soup findAllTags: 'table') size = 2.
	self assert: (soup table findAllTags: 'table') size = 1.
	self assert: (soup findTag: [:e | e name = 'table' and: [e id = '2']]) parent parent parent name = 'table'.
	soup := Soup fromString: '<table><tr><td><div><table>Foo</table></div></td></tr></table>'.
	self assert: soup table tr td div table string = 'Foo'.
	soup := Soup fromString: '<table><thead><tr>Foo</tr></thead><tbody><tr>Bar</tr></tbody>
        <tfoot><tr>Baz</tr></tfoot></table>'.
	self assert: soup table thead tr string = 'Foo'.! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/23/2008 04:54'!
testParaInsideBlockquote
	| soup |
	soup := Soup fromString: '<blockquote><p><b>Foo</blockquote><p>Bar'.
	self assert: soup blockquote p b string = 'Foo'.
	self assert: soup blockquote b string = 'Foo'.
	self assert: (soup findChildTag: 'p') string = 'Bar'! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/26/2008 22:26'!
testProcessingInstruction
	| soup data |
	soup := Soup fromString: 'abc<?pi>def'.
	data := soup findString: [:e | e isProcessingInstruction].
	self assert: data contents = 'pi'.
	self assert: data printString = '<?pi>'.
	self assert: data next contents = 'def'.! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/24/2008 17:39'!
testQuotedAttributeValues
	| text soup |
	self 
		assertSoupFrom: '<foo attr=''bar''></foo>' 
		printsAs: '<foo attr="bar"></foo>'.
	self 
		assertSoupFrom: '<foo attr=''bar "brawls" happen''>a</foo>' 
		printsAs: '<foo attr=''bar "brawls" happen''>a</foo>'.
		
	text := '<foo attr=''bar "brawls" happen''>a</foo>'.
	soup := Soup fromString: text.
	self assert: soup printString = text.
	soup foo attributeAt: 'attr' put: 'Brawls happen at "Bob''s Bar"'.
	self assert: soup printString = '<foo attr=''Brawls happen at "Bob&squot;s Bar"''>a</foo>'.
	
	self 
		assertSoupFrom: '<this is="really messed up & stuff">' 
		printsAs: '<this is="really messed up &amp; stuff"></this>'.
	
	" This is not what the original author had in mind, 
	but it's a legitimate interpretation of what they wrote. "
	self 
		assertSoupFrom: '<a href="foo</a>, </a><a href="bar">baz</a>' 
		printsAs: '<a href="foo&lt;/a&gt;, &lt;/a&gt;&lt;a href="></a>, <a href="bar">baz</a>'.

	" SGMLParser generates bogus parse events when attribute values
	contain embedded brackets, but at least Beautiful Soup fixes
	it up a little. "
	self 
		assertSoupFrom: '<a b="<a>">'
		printsAs: '<a b="&lt;a&gt;"></a><a>"></a>' .
	self 
		assertSoupFrom: '<a href="http://foo.com/<a> and blah and blah'
		printsAs: '<a href=''"http://foo.com/''></a><a> and blah and blah</a>'.
! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/23/2008 06:09'!
testSelfClosingTag
	self assert: ((Soup fromString: 'Foo<br/>Bar') findTag: 'br') printString = '<br />'.
	self assert: (Soup fromString: '<p>test1<br/>test2</p>') printString = (Soup fromString: '<p>test1<br />test2</p>') printString.
	self assert: (Soup fromString: '<p>test1<selfclosing>test2') printString = '<p>test1<selfclosing>test2</selfclosing></p>'.
	! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/23/2008 02:45'!
testTextArea
	| soup |
	soup := Soup fromString: '<textarea><b>This is an example of an HTML tag</b><&<&</textarea>'.
	self assert: soup textarea string = '<b>This is an example of an HTML tag</b><&<&'! !

!SoupParserTest methodsFor: 'tests' stamp: 'za 12/24/2008 19:47'!
testWhitespaceInDeclaration
	self 
		assertSoupFrom: '<!! DOCTYPE>'
		printsAs: '<!!DOCTYPE>'! !

!SoupParserTest methodsFor: 'tests' stamp: 'sd 12/11/2010 16:52'!
todotestNonBreakingSpaces

	self halt.
! !
